﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Airbus
{
    public class DrawningPlane
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityPlane Plane { get; protected set; }

        /// <summary>
        /// Левая координата отрисовки самолета
        /// </summary>
        protected float _startPosX;
        /// <summary>
        /// Верхняя кооридната отрисовки самолета
        /// </summary>
        protected float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        

        /// <summary>
        /// Ширина отрисовки самолета
        /// </summary>
        private readonly int _PlaneWidth = 130;
        /// <summary>
        /// Высота отрисовки самолета
        /// </summary>
        private readonly int _PlaneHeight = 70;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес самолета</param>
        /// <param name="bodyColor">Цвет кузова</param>
        public DrawningPlane(int speed, float weight, Color bodyColor)
        {
            Plane = new EntityPlane(speed, weight, bodyColor);
            
        }
        /// <summary>
        /// Установка позиции самолета
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void SetPosition(int x, int y, int width, int height)
        {
            if (x >= 0 && x + _PlaneWidth <= width && y >= 0 && y + _PlaneHeight <= height)
            {
                _startPosX = x;
                _startPosY = y;
                _pictureWidth = width;
                _pictureHeight = height;
            }
            
        }
        /// <summary>
        /// Изменение направления пермещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _PlaneWidth + Plane.Step < _pictureWidth)
                    {
                        _startPosX += Plane.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    if (_startPosX - Plane.Step > 0)
                    {
                        _startPosX -= Plane.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - Plane.Step > 0)
                    {
                        _startPosY -= Plane.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _PlaneHeight + Plane.Step < _pictureHeight)
                    {
                        _startPosY += Plane.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Отрисовка самолета
        /// </summary>
        /// <param name="g"></param>
        /// 
        protected DrawningPlane(int speed, float weight, Color bodyColor, int planeWidth, int planeHeight) : this(speed, weight, bodyColor)
        {
            _PlaneWidth = planeWidth;
            _PlaneHeight = planeHeight;
        }
        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
                || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black);
            //границы самолета
            g.DrawEllipse(pen, _startPosX, _startPosY + 30, 20, 20);
            g.DrawRectangle(pen, _startPosX + 10, _startPosY + 30, 100, 20);

            g.DrawLine(pen, _startPosX + 110, _startPosY + 30, _startPosX + 130, _startPosY+40);
            g.DrawLine(pen, _startPosX + 110, _startPosY+50, _startPosX + 130, _startPosY+40);


            g.DrawLine(pen, _startPosX, _startPosY, _startPosX, _startPosY+40);
            g.DrawLine(pen, _startPosX, _startPosY, _startPosX + 30, _startPosY+30);


            g.DrawLine(pen, _startPosX + 40, _startPosY + 50, _startPosX + 40, _startPosY+55);
            g.DrawLine(pen, _startPosX + 100, _startPosY + 50, _startPosX + 100, _startPosY+55);

            g.DrawEllipse(pen, _startPosX + 95, _startPosY + 55, 10, 10);
            g.DrawEllipse(pen, _startPosX + 29, _startPosY + 55, 10, 10);
            g.DrawEllipse(pen, _startPosX + 41, _startPosY + 55, 10, 10);

            Brush br = new SolidBrush(Plane?.BodyColor ?? Color.Black);
            g.FillEllipse(br, _startPosX, _startPosY + 31, 20, 19);
            g.FillRectangle(br, _startPosX + 10, _startPosY + 31, 100, 19);

            //илюминатор
            Brush brBlack = new SolidBrush(Color.Black);
            g.FillEllipse(brBlack, _startPosX + 40, _startPosY + 35, 60, 5);
            g.FillEllipse(brBlack, _startPosX - 5, _startPosY + 25, 30, 10);
        }
        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _PlaneWidth || _pictureHeight <= _PlaneHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _PlaneWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _PlaneWidth;
            }
            if (_startPosY + _PlaneHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _PlaneHeight;
            }
        }

        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return (_startPosX, _startPosY, _startPosX + _PlaneWidth, _startPosY + _PlaneHeight);
        }
    }
}
