﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Airbus
{
    internal class SetPlanesGeneric<T>
        where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetPlanesGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="plane">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T plane)
        {
            // TODO вставка в начало набора
            // TODO проверка на _maxCount
            if (_places.Count + 1 >= _maxCount)
                return -1;
            _places.Insert(0, plane);
            return 0;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="plane">Добавляемый автомобиль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T plane, int position)
        {
            // TODO проверка позиции
            // TODO вставка по позиции
            if (position >= _maxCount || position < 0)
            {
                return -1;
            }
            if (_places.Count + 1 >= _maxCount)
                return -1;
            _places.Insert(position, plane);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            // TODO проверка позиции
            // TODO удаление объекта из массива, присовив элементу массива значение null
            if (position >= _maxCount || position < 0)
            {
                return null;
            }   
            T DeletePlane = _places[position];
            _places.RemoveAt(position);
            return DeletePlane;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                // TODO проверка позиции
                if (position < 0 || position >= _maxCount)
                {
                    return null;
                }   
                return _places[position];
            }
            set
            {
                // TODO проверка позиции
                // TODO вставка в список по позиции
                if (position < 0 || position >= _maxCount)
                {
                    Insert(value, position);
                }   
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetPlanes()
        {
            foreach (var plane in _places)
            {
                if (plane != null)
                {
                    yield return plane;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
